"""
Quick test for the complex

"""
from pyphonebook.tests.conftest import parse_fake_query, candidates_matching_query


def test_query_parser_simple_statement():
    query = parse_fake_query("(cn=user1)")
    assert query == ["cn=user1"]


def test_query_parser_redundant_brackets():
    query = parse_fake_query("((cn=user1))")
    assert query == [["cn=user1"]]


def test_query_parser_simple_and_statement():
    query = parse_fake_query("(&(cn=user1))")
    assert query == ["&", ["cn=user1"]]


def test_query_parser_complex_and_statement():
    query = parse_fake_query("(&(cn=user1)(office=999 2A-22))")
    assert query == ["&", ["cn=user1"], ["office=999 2A-22"]]


def test_query_parser_simple_or_statement():
    query = parse_fake_query("(|(cn=user1))")
    assert query == ["|", ["cn=user1"]]


def test_query_parser_complex_or_statement():
    query = parse_fake_query("(|(cn=user1)(office=999 2A-22))")
    assert query == ["|", ["cn=user1"], ["office=999 2A-22"]]


def test_query_parser_nested_and_statement():
    query = parse_fake_query("(&(cn=user1)(&(sn=One)(office=999 2A-22)))")
    assert query == ["&", ["cn=user1"], ["&", ["sn=One"], ["office=999 2A-22"]]]


def test_query_parser_nested_or_statement():
    query = parse_fake_query("(|(cn=user1)(|(sn=One)(office=999 2A-22)))")
    assert query == ["|", ["cn=user1"], ["|", ["sn=One"], ["office=999 2A-22"]]]


def test_query_parser_deeply_nested_statement():
    query = parse_fake_query("(&(|(cn=user1)(|(sn=One)(&(office=999 2A-22)(cn=user3)))))")
    assert query == ["&", ["|", ["cn=user1"], ["|", ["sn=One"], ["&", ["office=999 2A-22"], ["cn=user3"]]]]]


def test_match_candidates_by_cn():
    results = candidates_matching_query(["cn=user1"])
    assert len(results) == 1
    assert results[0][1]["cn"] == [b"user1"]


def test_match_candidates_by_other_field():
    results = candidates_matching_query(["sn=Two"])
    assert len(results) == 1
    assert results[0][1]["cn"] == [b"user2"]


def test_match_candidates_multiple_results():
    results = candidates_matching_query(["division=BE"])
    assert len(results) == 2
    assert results[0][1]["cn"] == [b"user1"]
    assert results[1][1]["cn"] == [b"user2"]


def test_match_candidates_all_results():
    results = candidates_matching_query(["givenName=User"])
    assert len(results) == 3
    assert results[0][1]["cn"] == [b"user1"]
    assert results[1][1]["cn"] == [b"user2"]
    assert results[2][1]["cn"] == [b"user3"]


def test_match_candidates_no_results():
    results = candidates_matching_query(["division=WW"])
    assert len(results) == 0


def test_match_candidates_and():
    results = candidates_matching_query(["&", ["division=BE"], ["sn=Two"]])
    assert len(results) == 1
    assert results[0][1]["cn"] == [b"user2"]


def test_match_candidates_or():
    results = candidates_matching_query(["|", ["cn=user1"], ["sn=Two"]])
    assert len(results) == 2
    assert results[0][1]["cn"] == [b"user1"]
    assert results[1][1]["cn"] == [b"user2"]


def test_match_candidates_nested():
    results = candidates_matching_query(["&", ["division=BE"], ["|", ["cn=user1"], ["sn=Three"]]])
    assert len(results) == 1
    assert results[0][1]["cn"] == [b"user1"]