"""
High-level tests for the  package.

"""
import sys
from pyphonebook import _backport_cli, PhoneBookEntry
from pyphonebook.tests.conftest import FAKE_ENTRY_RAW_DATA_1


# TODO functions like merge_by_ccid(), arbitrary_search() and build_arbitrary_query() are not tested yet


def test_main_instantiate_cli():
    sys.argv = ["pyphonebook", "user3"]  # Some args just to make it run
    _backport_cli._cli()


def test_phonebook_raw_search(phonebook):
    results = phonebook._raw_search("(cn=user1)")
    assert len(results) == 1


def test_phonebook_search_raw_false(phonebook):
    results = phonebook.search(query="(cn=user1)", raw=False)
    [print(r) for r in results]
    assert len(results) == 1
    assert isinstance(results[0], PhoneBookEntry)


def test_phonebook_search_raw_true(phonebook):
    results = phonebook.search(query="(cn=user1)", raw=True)
    assert len(results) == 1
    assert isinstance(results[0], tuple)


def test_phonebook_search_by_ldap_parameter(phonebook):
    results = phonebook.search_by_ldap_parameter("sn", "Two")
    assert len(results) == 1
    assert results[0].login_name[0] == "user2"
    assert results[0].last_name[0] == "Two"


def test_phonebook_compound_search(phonebook):
    params = {"department": "BE", "preferredLanguage": "EN"}
    results = phonebook.multi_field_search(params)
    assert len(results) == 1
    assert results[0].login_name[0] == "user1"

    params = {"department": "BE", "preferredLanguage": "FR"}
    results = phonebook.multi_field_search(params)
    assert len(results) == 1
    assert results[0].login_name[0] == "user2"


def test_phonebook_validate_login_name(phonebook):
    assert phonebook.validate_login_name("user1")
    assert not phonebook.validate_login_name("user5")


def test_phonebook_search_by_login_name(phonebook):
    results = phonebook.search_by_login_name("user1")
    assert len(results) == 1
    assert results[0].login_name[0] == "user1"

    results = phonebook.search_by_login_name("user5")
    assert len(results) == 0


def test_phonebook_search_by_full_name(phonebook):
    results = phonebook.search_by_full_name("User Three")
    assert len(results) == 1
    assert results[0].login_name[0] == "user3"

    results = phonebook.search_by_login_name("User Five")
    assert len(results) == 0


def test_phonebook_search_by_first_name(phonebook):
    results = phonebook.search_by_first_name("User")
    assert len(results) == 3
    assert results[0].login_name[0] == "user1"
    assert results[1].login_name[0] == "user2"
    assert results[2].login_name[0] == "user3"

    results = phonebook.search_by_first_name("Another")
    assert len(results) == 0


def test_phonebook_search_by_last_name(phonebook):
    results = phonebook.search_by_last_name("Two")
    assert len(results) == 1
    assert results[0].login_name[0] == "user2"

    results = phonebook.search_by_last_name("Five")
    assert len(results) == 0


def test_phonebook_search_by_email(phonebook):
    results = phonebook.search_by_email("user.three@cern.ch")
    assert len(results) == 1
    assert results[0].login_name[0] == "user3"

    results = phonebook.search_by_email("user.five@cern.ch")
    assert len(results) == 0


def test_phonebook_search_by_phone_number(phonebook):
    results = phonebook.search_by_phone_number("1234567")
    assert len(results) == 1
    assert results[0].login_name[0] == "user3"

    results = phonebook.search_by_phone_number("0000000")
    assert len(results) == 0


def test_phonebook_search_by_landline(phonebook):
    # Add landline info to some user
    results = phonebook.search_by_landline("1234567")
    assert len(results) == 0


def test_phonebook_search_by_fax(phonebook):
    # Add fax info to some user
    results = phonebook.search_by_full_name("0987654")
    assert len(results) == 0


def test_phonebook_search_by_mobile(phonebook):
    # Add mobile info to some user
    results = phonebook.search_by_mobile("User Three")
    assert len(results) == 0


def test_phonebook_search_by_post_box(phonebook):
    results = phonebook.search_by_post_box("Z00010")
    assert len(results) == 1
    assert results[0].login_name[0] == "user1"

    results = phonebook.search_by_post_box("Z000001")
    assert len(results) == 0


def test_phonebook_search_by_building(phonebook):
    results = phonebook.search_by_building("999")
    assert len(results) == 3
    assert results[0].login_name[0] == "user1"
    assert results[1].login_name[0] == "user2"
    assert results[2].login_name[0] == "user3"

    results = phonebook.search_by_building("111")
    assert len(results) == 0


def test_phonebook_search_by_office(phonebook):
    results = phonebook.search_by_office("999 2-A22")
    assert len(results) == 1
    assert results[0].login_name[0] == "user2"

    results = phonebook.search_by_office("111 1-B11")
    assert len(results) == 0


def test_phonebook_search_by_department(phonebook):
    results = phonebook.search_by_department("BE")
    assert len(results) == 2
    assert results[0].login_name[0] == "user1"
    assert results[1].login_name[0] == "user2"

    results = phonebook.search_by_department("BEE")
    assert len(results) == 0


def test_phonebook_search_by_group(phonebook):
    results = phonebook.search_by_group("DT")
    assert len(results) == 1
    assert results[0].login_name[0] == "user3"

    results = phonebook.search_by_group("BE")
    assert len(results) == 0


def test_phonebook_search_by_section(phonebook):
    results = phonebook.search_by_section("APS")
    assert len(results) == 1
    assert results[0].login_name[0] == "user2"

    results = phonebook.search_by_section("EE")
    assert len(results) == 0


def test_phonebook_search_by_organizational_unit(phonebook):
    results = phonebook.search_by_organizational_unit("BE", "CO", "APS")
    assert len(results) == 1
    assert results[0].login_name[0] == "user2"

    results = phonebook.search_by_organizational_unit("BE", "CO", "DS")
    assert len(results) == 0


def test_phonebook_search_by_employee_id(phonebook):
    results = phonebook.search_by_employee_id("654322")
    assert len(results) == 1
    assert results[0].login_name[0] == "user3"

    results = phonebook.search_by_employee_id("000000")
    assert len(results) == 0


def test_phonebook_search_by_ccid(phonebook):
    results = phonebook.search_by_ccid("654322")
    assert len(results) == 1
    assert results[0].login_name[0] == "user3"

    results = phonebook.search_by_ccid("000000")
    assert len(results) == 0


def test_phonebook_search_by_uid(phonebook):
    results = phonebook.search_by_uid("223456")
    assert len(results) == 1
    assert results[0].login_name[0] == "user2"

    results = phonebook.search_by_uid("222222")
    assert len(results) == 0


def test_phonebook_search_by_gid(phonebook):
    results = phonebook.search_by_gid("2234")
    assert len(results) == 1
    assert results[0].login_name[0] == "user2"

    results = phonebook.search_by_gid("9999")
    assert len(results) == 0


def test_phonebook_search_by_home_directory(phonebook):
    results = phonebook.search_by_home_directory("/afs/cern.ch/user/u/user1")
    assert len(results) == 1
    assert results[0].login_name[0] == "user1"

    results = phonebook.search_by_home_directory("/afs/cern.ch/user/u/user9")
    assert len(results) == 0


# ##################################### #
#      Tests for PhoneBookEntry         #
# ##################################### #

def test_phonebookentry_valid(mock_users):
    for user in mock_users:
        entry = PhoneBookEntry(user)
        assert entry.parsing_succeeded
        assert "exception_raised" in vars(entry).keys()
        assert entry.exception_raised == []
        assert "raw_data" in vars(entry).keys()
        assert entry.to_json()
        assert str(entry)
        assert entry.login_name[0] == user[1]["cn"][0].decode()


def test_phonebookentry_unexpected_but_valid():
    entry = PhoneBookEntry(("broken", {"unexpected": "value"}))
    assert entry.parsing_succeeded
    assert "exception_raised" in vars(entry).keys()
    assert entry.exception_raised == []
    assert "raw_data" in vars(entry).keys()
    assert entry.to_json()
    assert str(entry)
    assert entry.login_name == []


def test_phonebookentry_broken():
    entry = PhoneBookEntry("I should not be a string!")
    assert not entry.parsing_succeeded
    assert "exception_raised" in vars(entry).keys()
    assert entry.exception_raised
    assert "raw_data" in vars(entry).keys()
    assert entry.to_json()
    assert str(entry)
    assert "login_name" not in vars(entry).keys()


# TODO test long_phone_number_to_short() better
def test_phonebookentry_long_phone_number_to_short():
    entry = PhoneBookEntry(FAKE_ENTRY_RAW_DATA_1)
    assert "41227697554" == entry.long_phone_number_to_short("41227697554")


# TODO test short_phone_number_to_long() better
def test_phonebookentry_short_phone_number_to_long():
    entry = PhoneBookEntry(FAKE_ENTRY_RAW_DATA_1)
    assert "41227697554" == entry.short_phone_number_to_long("41227697554")


